/* 
 *           Examplefile for usage of SAPI-Library 
 *
 *      This example shows how create own SURF-Data 
 *      for Multibeam-Sensors.
 *      You should also be sure that you are writing the same
 *      SURF-Version as your Offline-system is using.
 *      You may inspect the Version of this Library by
 *           typing :       what libsapi.a
 *      For 'reading only' any SURF-Version equal or higher than
 *      your system's version is o.k. , for writing it must
 *      be the same !!!!
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "sapi.h"

/* ERRORPRINT != 0 will force erroroutputs on stderr, if something is going wrong
                   during filehandling                                           */
                   
#define ERRORPRINT 1




/* Some mathematical macros */ 

#ifndef PI
#define PI 3.14159265359
#endif

#define DEG_TO_RAD(a)        ((double)((PI*a)/180.0) )

#define M_PER_RAD_LAT      ((double)(60.0*180.0*1852.0/PI))
#define M_PER_RAD_LON(LAT) (((double)(60.0*180.0*1852.0/PI))*cos(LAT))

/* simple flat projection */
#define M_TO_RAD_Y(LAT)      ((double)(LAT/M_PER_RAD_LAT))
#define M_TO_RAD_X(LON,LAT)  ((double)(LON/(M_PER_RAD_LON(LAT))))


char surfDataPath[256] = {""}; 
char filename[256] = {""}; 



int main (int argc,char ** argv)
{
 SurfGlobalData*               toGlobalData;
 SurfMultiBeamAngleTable*      toAngles;
 SurfTransducerParameterTable* surfTransducerTable;
 SurfPositionAnySensor*        surfPositionSensor;
 SurfMultiBeamTT*              toTT;
 SurfMultiBeamReceive*         toReceive;
 SurfSoundingData*             toSounding;
 SurfCenterPosition*           toPositions;
 SurfSidescanData*             toSs;
 
 char* sp;
 int n,sounding,nrSoundings;
 long status,ii,nrBeams,maxNrSidescanSamplesPerSounding;
 double travelTime,depth,posX,posY,relTime,angle;
 u_char ampl;

 posX=posY=0.0;
 relTime=0.0;
 depth = 20.0;
 

 /* Get Path and Filename (without extention '.six or .sda') from shell */

 for(n=1;n<argc;n++)
 {
  if((sp = strstr(argv[n],"-F")) != NULL) strncpy(filename,sp+2,255);
  if((sp = strstr(argv[n],"-D")) != NULL) strncpy(surfDataPath,sp+2,255);
 }

 if(*filename == '\0')sprintf(filename,"newBuildSurfFile");
 if(*surfDataPath == '\0')
#ifdef _WIN32
	 sprintf(surfDataPath,"../../data");
#else
     sprintf(surfDataPath,"../data");
#endif
 /* Maybe you like to printout actual Library-Versions */

 SAPI_printAPIandSURFversion();
  

 /* SAPI_createSurfBody creates an empty SURF-file, which may be filled 
    by your data . All records are accessible in memory 
    for writing them back to a file  */

 /* Setting the parameters for your data */

 /* You have got a collection of 279 pings into the water */    

 nrSoundings = 279;

 /* Each ping is received by 40 beams */

 nrBeams = 40;   /* nrBeams = 0 ==> Vertical Sounder */

 /* You are holding a maximum of 5000 SIDESCAN samples per ping */

 maxNrSidescanSamplesPerSounding = 5000;
                 /* maxNrSidescanSamplesPerSounding = 0
                    ==> no SIDESCAN data               */
                    
 if(SAPI_createSurfBody(nrSoundings,nrBeams,
                maxNrSidescanSamplesPerSounding,ERRORPRINT) == 0)
 {
  /* At first fill in some Global data : */

  if((toGlobalData = SAPI_getGlobalData()) != NULL)
  {
   /* Enter the ship's name (max. 15 characters) */

   sprintf(toGlobalData->shipsName,"Titanic III");

   /* Enter the start-time of your profile (DDMMYYHHMMSS.NN) */
   /* All times in the profile will be related to this value */

   sprintf(toGlobalData->startTimeOfProfile,"081298123000.00");

   /* Two strings may be used for Dokumentation: (max. 15 characters each) */ 

   sprintf(toGlobalData->regionOfProfile,"MARIANA TROUGH");
   sprintf(toGlobalData->numberOfProfile,"0001");

   /* You may set the name of your sounder: (max. 15 characters) */ 

   sprintf(toGlobalData->nameOfSounder,"MY_SOUNDER");

   /* You should set the frequencylayer of your data by entering the 
      frequency in Hz to one of 3 layers:
                 Low      < 30000Hz
      30000Hz <= Medium  =< 70000Hz
                 High     > 70000Hz 
   */

   toGlobalData->lowFrequency=12000.0;

   /* Enter the start-position of your profile in rad WGS84      */
   /* All positions in the profile will be related to this value */

   toGlobalData->referenceOfPositionX=DEG_TO_RAD(125.035); /*longitude 125.035 East */ 
   toGlobalData->referenceOfPositionY=DEG_TO_RAD(-17.352); /*latitude   17.352 South*/ 



   /* Set the beam angle table beam nr. 0 is the outermost on portside 
      angles on portside are negativ, on starside positive            
      Simulate equalspaced angles over 90deg                          */
               
   toAngles = SAPI_getActualAngleTable();
   for(ii=0;ii<nrBeams;ii++)
   {
    angle = DEG_TO_RAD(-45.0) + ((DEG_TO_RAD(90.0)) * (double)ii / (double)(nrBeams-1));
    toAngles->beamAngle[ii] = (float) angle;
   }



   /* Set the Transducer table entries                               */
   /* The Tranducer table describes the position offset [m] of the 
      transducer to that point where center-positions are related to, 
      transducerDepth is the draught of the transducer [m]           
      transducerTwoThetaLM/HFreq is the 2-Theta-angle of the
      transmitbeam                                                   */
      
   surfTransducerTable = SAPI_getActualTransducerTable();

   surfTransducerTable->transducerDepth = (float) 2.40;
   surfTransducerTable->transducerPositionAhead = (float) 5.30;
   surfTransducerTable->transducerPositionStar = (float) -0.30;
   
   surfTransducerTable->transducerTwoThetaLFreq = (float) DEG_TO_RAD(2.0);



   /* Set the Position sensor table entries       */
   /* positionsensor's name  (max. 15 characters) */

   surfPositionSensor = SAPI_getPositionSensor(0l);

   sprintf(surfPositionSensor->positionSensorName,"myGPS");



   /* Fill the sounding data */

   sounding = 0;
   status = 0;
   while(status == 0)    /* This loop is stepping sounding by sounding */
   {
    toSounding = SAPI_getSoundingData();
    toAngles = SAPI_getActualAngleTable();
     
    /* times are relative to toGlobalData->startTimeOfProfile ,
       we are simulating here one ping every second */

    toSounding->relTime = (float)relTime;
    relTime = relTime + 1.0;

    /* tide-values may be attached, if known already in [m] */
                 
    toSounding->tide = (float) 1.23;

    /* the ships heading ( not the course !!) in [rad] */

    toSounding->headingWhileTransmitting = (float) DEG_TO_RAD(45.0);

    /* the heave/roll/pitch-values during transmission [m,rad,rad] */
    /* positive = heave: ship up , pitch: bow up , roll: port up   */

    toSounding->heaveWhileTransmitting = (float) 0.23;
    toSounding->rollWhileTransmitting  = (float) DEG_TO_RAD(0.2);
    toSounding->pitchWhileTransmitting = (float) DEG_TO_RAD(0.1);

    /* cKeel is the Soundvelocity at the transducer ,
       cMean is the representative Soundvelocity over the whole way */

    toSounding->cKeel = (float) 1493.2;
    toSounding->cMean = (float) 1493.2;

    /* Positiondata are relative to 'toGlobalData->referenceOfPositionX,Y' */    
    /* we are simulating here a NE-course with about 2m/sec                */

    toPositions=SAPI_getCenterPosition(0);
    toPositions->centerPositionX = (float)posX;
    toPositions->centerPositionY = (float)posY;
    posX = posX + M_TO_RAD_X(2.0,toGlobalData->referenceOfPositionY);
    posY = posY + M_TO_RAD_Y(2.0);
    toPositions->speed = (float) 2.0;


    for(ii=0;ii<nrBeams;ii++)
    {
     /* Simulating traveltimes for depth starting at 20m  and increasing
       (one way !!) in [sec] */
        
     toTT=SAPI_getMultiBeamTraveltime((long)ii);
     angle = (double) toAngles->beamAngle[ii];
     travelTime = fabs(depth / (cos(angle) * 1493.2));
     toTT->travelTimeOfRay = (float) travelTime;

     depth=depth+0.002; /* sim. increasing depth */

     /* Setting motion data at receive-time of the different beams 
        if not known, it should be the same as the transmit-data   */ 

     toReceive = SAPI_getMultiBeamReceiveParams((long)ii);   
     toReceive->headingWhileReceiving = (float) DEG_TO_RAD(45.0);
     toReceive->heaveWhileReceiving   = (float) 0.23;

    }

    if(maxNrSidescanSamplesPerSounding>0)
    {
     if((toSs=SAPI_getSidescanData())!=NULL)
     {
      toSs->actualNrOfSsDataPort = (u_short)(maxNrSidescanSamplesPerSounding/2);
      toSs->actualNrOfSsDataStb  = (u_short)(maxNrSidescanSamplesPerSounding/2);


      /* Setting the Sidescan-Samplegates */
      
      toTT=SAPI_getMultiBeamTraveltime((long)(nrBeams/2));
      travelTime = (double)toTT->travelTimeOfRay;
      toSs->minSsTimePort=(float)(2.0*travelTime);       
      toSs->minSsTimeStb =(float)(2.0*travelTime);       
      toTT=SAPI_getMultiBeamTraveltime((long)(0));
      travelTime = (double)toTT->travelTimeOfRay;
      toSs->maxSsTimePort=(float)(2.0*travelTime);       
      toSs->maxSsTimeStb =(float)(2.0*travelTime);

      /* Simulating Portside Sidescandata */

      ampl=0;
      for(ii=0;ii<toSs->actualNrOfSsDataPort;ii++)
      {
       toSs->ssData[ii] = ampl;
       ampl++;       
      }        

      /* Simulating starside Sidescandata */

      ampl=0;
      for(ii=toSs->actualNrOfSsDataPort;
          ii<toSs->actualNrOfSsDataPort + toSs->actualNrOfSsDataStb;ii++)
      {
       toSs->ssData[ii] = ampl;
       ampl--;       
      }
     } 
    }

    /* SAPI_nextSounding steps to the next sounding                                     */
    /* it reponds with 0 as long as data are readable and as long as EOF is not reached */
   
    status=SAPI_nextSounding(0);
    sounding++;
   }  

   /* Writing the modified data to a file 
      Most missing values will be calculated */

   if(SAPI_writeBackFromMemory(surfDataPath,filename,ERRORPRINT) == 0l)
   {
    fprintf(stderr,"\nExampledata written to:\n    '%s/%s'\n",surfDataPath,filename);
   }    
  }

  /* SAPI_close will close files and destroy heap-allocated structures */

  SAPI_close();
 }   
}



